@@ -16,8 +16,7 @@ import numpy as np
 from numpy.random import MT19937, Generator
 from scipy.optimize import OptimizeResult
 from fcmaes.cmaes import _check_bounds
-from fcmaes.cmaescpp import libcmalib
-from fcmaes.dacpp import call_back_type
+from fcmaes.decpp import mo_call_back_type, callback, libcmalib
 
 os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
@@ -27,7 +26,7 @@ def minimize(fun,
              input_sigma = 0.166, 
              popsize = 0, 
              max_evaluations = 100000, 
-             stop_fittness = None, 
+             stop_fitness = None, 
              rg = Generator(MT19937()),
              runid=0):
        
@@ -39,7 +38,7 @@ def minimize(fun,
     fun : callable
         The objective function to be minimized.
             ``fun(x, *args) -> float``
-        where ``x`` is an 1-D array with shape (n,) and ``args``
+        where ``x`` is an 1-D array with shape (dim,) and ``args``
         is a tuple of the fixed parameters needed to completely
         specify the function.
     bounds : sequence or `Bounds`, optional
@@ -47,16 +46,16 @@ def minimize(fun,
             1. Instance of the `scipy.Bounds` class.
             2. Sequence of ``(min, max)`` pairs for each element in `x`. None
                is used to specify no bound.
-    x0 : ndarray, shape (n,)
-        Initial guess. Array of real elements of size (n,),
-        where 'n' is the number of independent variables.  
-    input_sigma : ndarray, shape (n,) or scalar
+    x0 : ndarray, shape (dim,)
+        Initial guess. Array of real elements of size (dim,),
+        where 'dim' is the number of independent variables.  
+    input_sigma : ndarray, shape (dim,) or scalar
         Initial step size for each dimension.
     popsize = int, optional
         CMA-ES population size.
     max_evaluations : int, optional
         Forced termination after ``max_evaluations`` function evaluations.
-    stop_fittness : float, optional 
+    stop_fitness : float, optional 
          Limit for fitness value. If reached minimize terminates.
     rg = numpy.random.Generator, optional
         Random generator for creating random guesses.
@@ -75,47 +74,35 @@ def minimize(fun,
         ``success`` a Boolean flag indicating if the optimizer exited successfully. """
     
     lower, upper, guess = _check_bounds(bounds, x0, rg)      
-    n = guess.size   
+    dim = guess.size   
     if lower is None:
-        lower = [0]*n
-        upper = [0]*n
+        lower = [0]*dim
+        upper = [0]*dim
     if callable(input_sigma):
         input_sigma=input_sigma()
     if np.ndim(input_sigma) == 0:
-        input_sigma = [input_sigma] * n
-    if stop_fittness is None:
-        stop_fittness = -math.inf   
-    array_type = ct.c_double * n 
-    c_callback = call_back_type(callback(fun))
-    res = np.empty(n+4)
+        input_sigma = [input_sigma] * dim
+    if stop_fitness is None:
+        stop_fitness = -math.inf   
+    array_type = ct.c_double * dim 
+    c_callback = mo_call_back_type(callback(fun, dim))
+    res = np.empty(dim+4)
     res_p = res.ctypes.data_as(ct.POINTER(ct.c_double))
     try:
-        optimizeCsma_C(runid, c_callback, n, int(rg.uniform(0, 2**32 - 1)), 
+        optimizeCsma_C(runid, c_callback, dim, int(rg.uniform(0, 2**32 - 1)), 
                 array_type(*guess), array_type(*lower), array_type(*upper), 
-                array_type(*input_sigma), max_evaluations, stop_fittness, popsize, res_p)
-        x = res[:n]
-        val = res[n]
-        evals = int(res[n+1])
-        iterations = int(res[n+2])
-        stop = int(res[n+3])
+                array_type(*input_sigma), max_evaluations, stop_fitness, popsize, res_p)
+        x = res[:dim]
+        val = res[dim]
+        evals = int(res[dim+1])
+        iterations = int(res[dim+2])
+        stop = int(res[dim+3])
         return OptimizeResult(x=x, fun=val, nfev=evals, nit=iterations, status=stop, success=True)
     except Exception as ex:
         return OptimizeResult(x=None, fun=sys.float_info.max, nfev=0, nit=0, status=-1, success=False)
 
-class callback(object):
-    
-    def __init__(self, fun):
-        self.fun = fun
-    
-    def __call__(self, n, x):
-        try:
-            fit = self.fun([x[i] for i in range(n)])
-            return fit if math.isfinite(fit) else sys.float_info.max
-        except Exception:
-            return sys.float_info.max
-  
 optimizeCsma_C = libcmalib.optimizeCsma_C
-optimizeCsma_C.argtypes = [ct.c_long, call_back_type, ct.c_int, ct.c_int, \
+optimizeCsma_C.argtypes = [ct.c_long, mo_call_back_type, ct.c_int, ct.c_int, \
             ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), \
             ct.POINTER(ct.c_double), ct.c_int, ct.c_double, ct.c_int, ct.POINTER(ct.c_double)]
         
